package com.jjt.calc.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.jjt.common.annotation.Log;
import com.jjt.common.core.controller.BaseController;
import com.jjt.common.core.domain.AjaxResult;
import com.jjt.common.enums.BusinessType;
import com.jjt.calc.domain.TwinCalcDay;
import com.jjt.calc.service.ITwinCalcDayService;
import com.jjt.common.utils.poi.ExcelUtil;
import com.jjt.common.core.page.TableDataInfo;

/**
 * 日统计数据Controller
 *
 * @author wukai
 * @date 2025-01-18
 */
@Api(tags="日统计数据")
@RestController
@RequestMapping("/calc/calcDay")
public class TwinCalcDayController extends BaseController{
    @Resource
    private ITwinCalcDayService twinCalcDayService;

    /**
     * 查询日统计数据列表
     */
    @ApiOperation("查询日统计数据列表")
    @PreAuthorize("@ss.hasPermi('calc:calcDay:list')")
    @GetMapping("/list")
    public TableDataInfo list(TwinCalcDay twinCalcDay)
    {
        startPage();
        List<TwinCalcDay> list = twinCalcDayService.selectTwinCalcDayList(twinCalcDay);
        return getDataTable(list);
    }

    /**
     * 导出日统计数据列表
     */
    @ApiOperation("导出日统计数据列表")
    @PreAuthorize("@ss.hasPermi('calc:calcDay:export')")
    @Log(title = "日统计数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TwinCalcDay twinCalcDay)
    {
        List<TwinCalcDay> list = twinCalcDayService.selectTwinCalcDayList(twinCalcDay);
        ExcelUtil<TwinCalcDay> util = new ExcelUtil<TwinCalcDay>(TwinCalcDay.class);
        util.exportExcel(response, list, "日统计数据数据");
    }

    /**
     * 获取日统计数据详细信息
     */
    @ApiOperation("获取日统计数据详细信息")
    @PreAuthorize("@ss.hasPermi('calc:calcDay:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(twinCalcDayService.selectTwinCalcDayById(id));
    }

    /**
     * 新增日统计数据
     */
    @ApiOperation("新增日统计数据")
    @PreAuthorize("@ss.hasPermi('calc:calcDay:add')")
    @Log(title = "日统计数据", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TwinCalcDay twinCalcDay)
    {
        return toAjax(twinCalcDayService.insertTwinCalcDay(twinCalcDay));
    }

    /**
     * 修改日统计数据
     */
    @ApiOperation("修改日统计数据")
    @PreAuthorize("@ss.hasPermi('calc:calcDay:edit')")
    @Log(title = "日统计数据", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TwinCalcDay twinCalcDay)
    {
        return toAjax(twinCalcDayService.updateTwinCalcDay(twinCalcDay));
    }

    /**
     * 删除日统计数据
     */
    @ApiOperation("删除日统计数据")
    @PreAuthorize("@ss.hasPermi('calc:calcDay:remove')")
    @Log(title = "日统计数据", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(twinCalcDayService.deleteTwinCalcDayByIds(ids));
    }
}
